/**
 * Copyright (c) 2018 Gregorius Techneticies
 *
 * This file is part of GregTech.
 *
 * GregTech is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GregTech is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with GregTech. If not, see <http://www.gnu.org/licenses/>.
 */

package gregapi.render;

import static gregapi.data.CS.*;

import cpw.mods.fml.client.registry.ISimpleBlockRenderingHandler;
import gregapi.util.UT;
import net.minecraft.block.Block;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.util.IIcon;
import net.minecraft.util.MathHelper;
import net.minecraft.world.IBlockAccess;
import net.minecraftforge.fluids.BlockFluidBase;
import net.minecraftforge.fluids.IFluidBlock;

public class RendererBlockFluid implements ISimpleBlockRenderingHandler {
	public static int RENDER_ID = 0;
	public static RendererBlockFluid INSTANCE;
	
	public RendererBlockFluid(int aRenderID) {
		INSTANCE = this;
		RENDER_ID = aRenderID;
	}
	
	static final float LIGHT_Y_NEG = 0.5F;
	static final float LIGHT_Y_POS = 1.0F;
	static final float LIGHT_XZ_NEG = 0.8F;
	static final float LIGHT_XZ_POS = 0.6F;
	static final double RENDER_OFFSET = 0.0010000000474974513D;
	static final float MAX_FLUID_HEIGHT = 0.8888889F;
	
	public float getFluidHeightAverage(float[] aFlow) {
		float total = 0, end = 0;
		int count = 0;
		for (int i = 0; i < aFlow.length; i++) {
			if (aFlow[i] >= MAX_FLUID_HEIGHT && end != 1F) end = aFlow[i];
			if (aFlow[i] >= 0) {
				total += aFlow[i];
				count++;
			}
		}
		if (end == 0) end = total / count;
		return end;
	}
	
	public float getFluidHeightForRender(IBlockAccess aWorld, int aX, int aY, int aZ, BlockFluidBase aFluidBlock) {
		if (aWorld.getBlock(aX, aY, aZ) == aFluidBlock) {
			Block verticalOrigin = aWorld.getBlock(aX, aY - UT.Fluids.dir(aFluidBlock), aZ);
			if (verticalOrigin.getMaterial().isLiquid() || verticalOrigin instanceof IFluidBlock) return 1;
			if (aWorld.getBlockMetadata(aX, aY, aZ) >= aFluidBlock.getMaxRenderHeightMeta()) return MAX_FLUID_HEIGHT;
		}
		return !aWorld.getBlock(aX, aY, aZ).getMaterial().isSolid() && aWorld.getBlock(aX, aY - UT.Fluids.dir(aFluidBlock), aZ) == aFluidBlock ? 1 : aFluidBlock.getQuantaPercentage(aWorld, aX, aY, aZ) * MAX_FLUID_HEIGHT;
	}
	
	@Override public void renderInventoryBlock(Block block, int metadata, int modelID, RenderBlocks renderer) {/**/}
	
	@Override
	public boolean renderWorldBlock(IBlockAccess aWorld, int aX, int aY, int aZ, Block aBlock, int aModelID, RenderBlocks aRenderer) {
		if (!(aBlock instanceof BlockFluidBase)) return F;
		
		Tessellator tessellator = Tessellator.instance;
		int color = aBlock.colorMultiplier(aWorld, aX, aY, aZ);
		float red = (color >> 16 & 255) / 255.0F;
		float green = (color >> 8 & 255) / 255.0F;
		float blue = (color & 255) / 255.0F;
		
		BlockFluidBase aFluid = (BlockFluidBase)aBlock;
		int bMeta = aWorld.getBlockMetadata(aX, aY, aZ), aDir = UT.Fluids.dir(aFluid);
		
		boolean renderTop = aWorld.getBlock(aX, aY - aDir, aZ) != aFluid;

		boolean renderBottom = aBlock.shouldSideBeRendered(aWorld, aX, aY + aDir, aZ, 0) && aWorld.getBlock(aX, aY + aDir, aZ) != aFluid;

		boolean[] renderSides = new boolean[] {
			aBlock.shouldSideBeRendered(aWorld, aX, aY, aZ - 1, 2), 
			aBlock.shouldSideBeRendered(aWorld, aX, aY, aZ + 1, 3),
			aBlock.shouldSideBeRendered(aWorld, aX - 1, aY, aZ, 4), 
			aBlock.shouldSideBeRendered(aWorld, aX + 1, aY, aZ, 5)
		};

		if (!renderTop && !renderBottom && !renderSides[0] && !renderSides[1] && !renderSides[2] && !renderSides[3]) return F;
		boolean rendered = F;
		double heightNW, heightSW, heightSE, heightNE;
		float flow11 = getFluidHeightForRender(aWorld, aX, aY, aZ, aFluid);

		if (flow11 != 1) {
			float flow00 = getFluidHeightForRender(aWorld, aX - 1, aY, aZ - 1, aFluid);
			float flow01 = getFluidHeightForRender(aWorld, aX - 1, aY, aZ,     aFluid);
			float flow02 = getFluidHeightForRender(aWorld, aX - 1, aY, aZ + 1, aFluid);
			float flow10 = getFluidHeightForRender(aWorld, aX,     aY, aZ - 1, aFluid);
			float flow12 = getFluidHeightForRender(aWorld, aX,     aY, aZ + 1, aFluid);
			float flow20 = getFluidHeightForRender(aWorld, aX + 1, aY, aZ - 1, aFluid);
			float flow21 = getFluidHeightForRender(aWorld, aX + 1, aY, aZ,     aFluid);
			float flow22 = getFluidHeightForRender(aWorld, aX + 1, aY, aZ + 1, aFluid);

			heightNW = getFluidHeightAverage(new float[] {flow00, flow01, flow10, flow11});
			heightSW = getFluidHeightAverage(new float[] {flow01, flow02, flow12, flow11});
			heightSE = getFluidHeightAverage(new float[] {flow12, flow21, flow22, flow11});
			heightNE = getFluidHeightAverage(new float[] {flow10, flow20, flow21, flow11});
		} else {
			heightNW = flow11;
			heightSW = flow11;
			heightSE = flow11;
			heightNE = flow11;
		}

		boolean rises = aDir == 1;
		if (aRenderer.renderAllFaces || renderTop) {
			rendered = T;
			IIcon iconStill = aBlock.getIcon(1, bMeta);
			float flowDir = (float)BlockFluidBase.getFlowDirection(aWorld, aX, aY, aZ);
			
			if (flowDir > -999.0F) iconStill = aBlock.getIcon(2, bMeta);
			
			heightNW -= RENDER_OFFSET;
			heightSW -= RENDER_OFFSET;
			heightSE -= RENDER_OFFSET;
			heightNE -= RENDER_OFFSET;

			double u1, u2, u3, u4, v1, v2, v3, v4;

			if (flowDir < -999.0F) {
				u2 = iconStill.getInterpolatedU(0.0D);
				v2 = iconStill.getInterpolatedV(0.0D);
				u1 = u2;
				v1 = iconStill.getInterpolatedV(16.0D);
				u4 = iconStill.getInterpolatedU(16.0D);
				v4 = v1;
				u3 = u4;
				v3 = v2;
			} else {
				float xFlow = MathHelper.sin(flowDir) * 0.25F;
				float zFlow = MathHelper.cos(flowDir) * 0.25F;
				u2 = iconStill.getInterpolatedU(8.0F + (-zFlow - xFlow) * 16.0F);
				v2 = iconStill.getInterpolatedV(8.0F + (-zFlow + xFlow) * 16.0F);
				u1 = iconStill.getInterpolatedU(8.0F + (-zFlow + xFlow) * 16.0F);
				v1 = iconStill.getInterpolatedV(8.0F + (+zFlow + xFlow) * 16.0F);
				u4 = iconStill.getInterpolatedU(8.0F + (+zFlow + xFlow) * 16.0F);
				v4 = iconStill.getInterpolatedV(8.0F + (+zFlow - xFlow) * 16.0F);
				u3 = iconStill.getInterpolatedU(8.0F + (+zFlow - xFlow) * 16.0F);
				v3 = iconStill.getInterpolatedV(8.0F + (-zFlow - xFlow) * 16.0F);
			}

			tessellator.setBrightness(aBlock.getMixedBrightnessForBlock(aWorld, aX, aY, aZ));
			tessellator.setColorOpaque_F(LIGHT_Y_POS * red, LIGHT_Y_POS * green, LIGHT_Y_POS * blue);

			if (!rises) {
				tessellator.addVertexWithUV(aX + 0, aY + heightNW, aZ + 0, u2, v2);
				tessellator.addVertexWithUV(aX + 0, aY + heightSW, aZ + 1, u1, v1);
				tessellator.addVertexWithUV(aX + 1, aY + heightSE, aZ + 1, u4, v4);
				tessellator.addVertexWithUV(aX + 1, aY + heightNE, aZ + 0, u3, v3);
				
				tessellator.addVertexWithUV(aX + 0, aY + heightNW, aZ + 0, u2, v2);
				tessellator.addVertexWithUV(aX + 1, aY + heightNE, aZ + 0, u3, v3);
				tessellator.addVertexWithUV(aX + 1, aY + heightSE, aZ + 1, u4, v4);
				tessellator.addVertexWithUV(aX + 0, aY + heightSW, aZ + 1, u1, v1);
			} else {
				tessellator.addVertexWithUV(aX + 1, aY + 1 - heightNE, aZ + 0, u3, v3);
				tessellator.addVertexWithUV(aX + 1, aY + 1 - heightSE, aZ + 1, u4, v4);
				tessellator.addVertexWithUV(aX + 0, aY + 1 - heightSW, aZ + 1, u1, v1);
				tessellator.addVertexWithUV(aX + 0, aY + 1 - heightNW, aZ + 0, u2, v2);
				
				tessellator.addVertexWithUV(aX + 1, aY + 1 - heightNE, aZ + 0, u3, v3);
				tessellator.addVertexWithUV(aX + 0, aY + 1 - heightNW, aZ + 0, u2, v2);
				tessellator.addVertexWithUV(aX + 0, aY + 1 - heightSW, aZ + 1, u1, v1);
				tessellator.addVertexWithUV(aX + 1, aY + 1 - heightSE, aZ + 1, u4, v4);
			}
		}

		if (aRenderer.renderAllFaces || renderBottom) {
			rendered = T;
			tessellator.setBrightness(aBlock.getMixedBrightnessForBlock(aWorld, aX, aY - 1, aZ));
			if (!rises) {
				tessellator.setColorOpaque_F(LIGHT_Y_NEG * red, LIGHT_Y_NEG * green, LIGHT_Y_NEG * blue);
				aRenderer.renderFaceYNeg(aBlock, aX, aY + RENDER_OFFSET, aZ, aBlock.getIcon(0, bMeta));
			} else {
				tessellator.setColorOpaque_F(LIGHT_Y_POS * red, LIGHT_Y_POS * green, LIGHT_Y_POS * blue);
				aRenderer.renderFaceYPos(aBlock, aX, aY + RENDER_OFFSET, aZ, aBlock.getIcon(1, bMeta));
			}
		}

		for (int side = 0; side < 4; ++side) {
			int x2 = aX;
			int z2 = aZ;

			switch (side) {
				case 0: --z2; break;
				case 1: ++z2; break;
				case 2: --x2; break;
				case 3: ++x2; break;
			}

			IIcon iconFlow = aBlock.getIcon(side + 2, bMeta);
			if (aRenderer.renderAllFaces || renderSides[side]) {
				rendered = T;

				double ty1;
				double tx1;
				double ty2;
				double tx2;
				double tz1;
				double tz2;

				if (side == 0) {
					ty1 = heightNW;
					ty2 = heightNE;
					tx1 = aX;
					tx2 = aX + 1;
					tz1 = aZ + RENDER_OFFSET;
					tz2 = aZ + RENDER_OFFSET;
				} else if (side == 1) {
					ty1 = heightSE;
					ty2 = heightSW;
					tx1 = aX + 1;
					tx2 = aX;
					tz1 = aZ + 1 - RENDER_OFFSET;
					tz2 = aZ + 1 - RENDER_OFFSET;
				} else if (side == 2) {
					ty1 = heightSW;
					ty2 = heightNW;
					tx1 = aX + RENDER_OFFSET;
					tx2 = aX + RENDER_OFFSET;
					tz1 = aZ + 1;
					tz2 = aZ;
				} else {
					ty1 = heightNE;
					ty2 = heightSE;
					tx1 = aX + 1 - RENDER_OFFSET;
					tx2 = aX + 1 - RENDER_OFFSET;
					tz1 = aZ;
					tz2 = aZ + 1;
				}

				float u1Flow = iconFlow.getInterpolatedU(0.0D);
				float u2Flow = iconFlow.getInterpolatedU(8.0D);
				float v1Flow = iconFlow.getInterpolatedV((1.0D - ty1) * 16.0D * 0.5D);
				float v2Flow = iconFlow.getInterpolatedV((1.0D - ty2) * 16.0D * 0.5D);
				float v3Flow = iconFlow.getInterpolatedV(8.0D);
				tessellator.setBrightness(aBlock.getMixedBrightnessForBlock(aWorld, x2, aY, z2));
				float sideLighting = (side < 2?LIGHT_XZ_NEG:LIGHT_XZ_POS);

				tessellator.setColorOpaque_F(LIGHT_Y_POS * sideLighting * red, LIGHT_Y_POS * sideLighting * green, LIGHT_Y_POS * sideLighting * blue);

				if (!rises) {
					tessellator.addVertexWithUV(tx1, aY + ty1, tz1, u1Flow, v1Flow);
					tessellator.addVertexWithUV(tx2, aY + ty2, tz2, u2Flow, v2Flow);
					tessellator.addVertexWithUV(tx2, aY + 0, tz2, u2Flow, v3Flow);
					tessellator.addVertexWithUV(tx1, aY + 0, tz1, u1Flow, v3Flow);

					tessellator.addVertexWithUV(tx1, aY + ty1, tz1, u1Flow, v1Flow);
					tessellator.addVertexWithUV(tx1, aY + 0, tz1, u1Flow, v3Flow);
					tessellator.addVertexWithUV(tx2, aY + 0, tz2, u2Flow, v3Flow);
					tessellator.addVertexWithUV(tx2, aY + ty2, tz2, u2Flow, v2Flow);
				} else {
					tessellator.addVertexWithUV(tx1, aY + 1 - 0, tz1, u1Flow, v3Flow);
					tessellator.addVertexWithUV(tx2, aY + 1 - 0, tz2, u2Flow, v3Flow);
					tessellator.addVertexWithUV(tx2, aY + 1 - ty2, tz2, u2Flow, v2Flow);
					tessellator.addVertexWithUV(tx1, aY + 1 - ty1, tz1, u1Flow, v1Flow);

					tessellator.addVertexWithUV(tx1, aY + 1 - 0, tz1, u1Flow, v3Flow);
					tessellator.addVertexWithUV(tx1, aY + 1 - ty1, tz1, u1Flow, v1Flow);
					tessellator.addVertexWithUV(tx2, aY + 1 - ty2, tz2, u2Flow, v2Flow);
					tessellator.addVertexWithUV(tx2, aY + 1 - 0, tz2, u2Flow, v3Flow);
				}
			}
		}
		aRenderer.renderMinY = 0;
		aRenderer.renderMaxY = 1;
		return rendered;
	}
	
	@Override public boolean shouldRender3DInInventory(int modelId){ return F; }
	@Override public int getRenderId() {return RENDER_ID;}
}
